from unittest.mock import patch, call, MagicMock

import pytest
from ckan.plugins import toolkit


class MockSession(dict):
    def __init__(self):
        super().__init__()
        self.save = MagicMock()


@pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
@pytest.mark.ckan_config('ckan.plugins', 'twitter')
@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
def test_clear(app):
    package_id = u'some-package-id'
    url = toolkit.url_for('tweet.clear', package_id=package_id)

    mock_cache_helpers = MagicMock()
    mock_session = MockSession()
    mock_session['twitter_is_suitable'] = package_id

    with patch(u'ckanext.twitter.routes.tweet.session', mock_session):
        with patch(u'ckanext.twitter.routes.tweet.cache_helpers', mock_cache_helpers):
            app.post(url)

    assert mock_cache_helpers.remove_from_cache.call_args == call(package_id)
    assert 'twitter_is_suitable' not in mock_session
    assert mock_session.save.called


@pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
@pytest.mark.ckan_config('ckan.plugins', 'twitter')
@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
def test_clear_session_is_empty(app):
    package_id = u'some-package-id'
    url = toolkit.url_for('tweet.clear', package_id=package_id)

    mock_cache_helpers = MagicMock()
    mock_session = MockSession()

    with patch(u'ckanext.twitter.routes.tweet.session', mock_session):
        with patch(u'ckanext.twitter.routes.tweet.cache_helpers', mock_cache_helpers):
            app.post(url)

    assert mock_cache_helpers.remove_from_cache.call_args == call(package_id)
    assert 'twitter_is_suitable' not in mock_session
    assert not mock_session.save.called



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

import pytest

import ckanext.twitter.lib.config_helpers as config_helpers


@pytest.mark.ckan_config('ckanext.twitter.debug', True)
def test_gets_debug_value_when_present():
    assert config_helpers.twitter_is_debug()


@pytest.mark.ckan_config('ckanext.twitter.debug', False)
def test_gets_debug_value_when_present_even_if_false():
    assert not config_helpers.twitter_is_debug()


@pytest.mark.ckan_config('debug', True)
def test_gets_debug_default_when_absent():
    assert config_helpers.twitter_is_debug()


@pytest.mark.ckan_config('ckanext.twitter.hours_between_tweets', 2)
def test_gets_hours_between_tweets_value_when_present():
    assert config_helpers.twitter_hours_between_tweets() == 2


def test_gets_hours_between_tweets_default_when_absent():
    assert config_helpers.twitter_hours_between_tweets() == 24


@pytest.mark.ckan_config('ckanext.twitter.consumer_key', 'a-consumer-key')
@pytest.mark.ckan_config('ckanext.twitter.consumer_secret', 'a-consumer-secret')
@pytest.mark.ckan_config('ckanext.twitter.token_key', 'a-token-key')
@pytest.mark.ckan_config('ckanext.twitter.token_secret', 'a-token-secret')
def test_twitter_get_credentials():
    ck, cs, tk, ts = config_helpers.twitter_get_credentials()
    assert ck == 'a-consumer-key'
    assert cs == 'a-consumer-secret'
    assert tk == 'a-token-key'
    assert ts == 'a-token-secret'


def test_twitter_get_credentials_defaults():
    ck, cs, tk, ts = config_helpers.twitter_get_credentials()
    assert ck == 'no-consumer-key-set'
    assert cs == 'no-consumer-secret-set'
    assert tk == 'no-token-key-set'
    assert ts == 'no-token-secret-set'


@pytest.mark.ckan_config('ckanext.twitter.disable_edit', True)
def test_gets_disable_edit_value_when_present():
    assert config_helpers.twitter_disable_edit()


def test_gets_disable_edit_default_when_absent():
    assert not config_helpers.twitter_disable_edit()



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

import logging
from contextlib import suppress

import oauth2

from ckanext.twitter.lib import cache_helpers, config_helpers

logger = logging.getLogger('ckanext.twitter')


@cache_helpers.cache_manager.cache('twitter', 'client')
def twitter_client():
    """
    Attempts to create a client for accessing the twitter API using the credentials
    specified in the configuration file. Does not test for success. Caches the resulting
    client in the 'twitter' cache.

    :return: oauth2.Client
    """
    (
        consumer_key,
        consumer_secret,
        token_key,
        token_secret,
    ) = config_helpers.twitter_get_credentials()
    consumer = oauth2.Consumer(consumer_key, consumer_secret)
    token = oauth2.Token(token_key, token_secret)
    client = oauth2.Client(consumer, token)
    return client


def twitter_authenticate():
    """
    Verifies that the client is able to connect to the twitter API.

    Refreshes any unauthenticated cached client.
    :return: boolean
    """
    authenticated = False
    while not authenticated:
        client = twitter_client()
        url = 'https://api.twitter.com/1.1/account/verify_credentials.json'
        response, content = client.request(url, 'GET')
        authenticated = response.status == 200
        if not authenticated:
            # if the client isn't in the cache we don't care
            with suppress(KeyError):
                cache_helpers.cache_manager.invalidate(twitter_client)
            break
    if authenticated:
        if any(
            [
                c.startswith('no-') and c.endswith('-set')
                for c in config_helpers.twitter_get_credentials()
            ]
        ):
            authenticated = False
    return authenticated


def post_tweet(tweet_text, pkg_id):
    """
    Attempts to post the tweet. Returns a boolean success variable and a message
    describing the reason for the failure/success in posting the tweet.

    :param tweet_text: The text to post. This is passed in rather than
    generated inside the method to allow users to change the tweet before
    posting (if enabled).
    :param pkg_id: The package ID (for caching).
    :return: boolean, str
    """
    if config_helpers.twitter_is_debug():
        logger.debug(f'Not posted (debug): {tweet_text}')
        return False, 'debug'

    # if not enough time has passed since the last tweet
    if not cache_helpers.expired(pkg_id):
        logger.debug(f'Not posted (insufficient rest period): {tweet_text}')
        return False, 'insufficient rest period'

    # if we can't authenticate
    if not twitter_authenticate():
        logger.debug(f'Not posted (not authenticated): {tweet_text}')
        return False, 'not authenticated'

    # try to actually post
    client = twitter_client()
    url = 'https://api.twitter.com/1.1/statuses/update.json'
    params = {'status': tweet_text}
    request = oauth2.Request(method='POST', url=url, parameters=params)
    postdata = request.to_postdata()
    response, content = client.request(url, 'POST', postdata)
    if response.status == 200:
        cache_helpers.cache(pkg_id)
        logger.debug(f'Posted successfully: {tweet_text}')
    else:
        logger.debug(f'Not posted (tweet unsuccessful): {tweet_text}')
    return response.status == 200, f'{response.status} {response.reason}'



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

from ckan.common import session
from ckan.plugins import toolkit
from ckanext.twitter.lib import parsers as twitter_parsers


class TwitterJSHelpers:
    """
    A class defining various methods to pass into the templates as helpers.
    """

    def _get_package(self, package_name_or_id):
        """
        Gets the package dictionary.

        :param package_name_or_id: Preferably the package ID, but either the
        name or ID.
        :return: dict
        """
        return toolkit.get_action('package_show')({}, {'id': package_name_or_id})

    def _is_new(self, package_id):
        """
        Tests to see if the package is "new", i.e. it has only just had its first
        resource added.

        :param package_id: The ID of the package to check.
        :return: boolean
        """
        revisions = toolkit.get_action('package_activity_list')({}, {'id': package_id})
        return len(revisions) <= 3

    def tweet_ready(self, package_id):
        """
        Checks the session to see if the package has been marked as ready for tweeting
        via the update hook. Removes the 'twitter_is_suitable' key from the session if
        present.

        :param package_id: The package ID.
        :return: boolean
        """
        in_session = session.pop('twitter_is_suitable', '') == package_id
        return in_session

    def get_tweet(self, package_id):
        """
        Generates the tweet text for the given package.

        :param package_id: The package ID.
        :return: str
        """
        return twitter_parsers.generate_tweet({}, package_id, self._is_new(package_id))


def twitter_pkg_suitable(context, pkg_id, pkg_dict=None):
    """
    Various tests to determine if a package is suitable for tweeting about, e.g. it's
    active & has resources.

    :param context: The current context.
    :param pkg_id: The package ID.
    :param pkg_dict: Optionally provided instead of pkg_id to avoid requesting from the API. Not
                     recommended - provided as a test helper.
    :return: boolean
    """
    if pkg_dict:
        package = pkg_dict
    else:
        try:
            package = toolkit.get_action('package_show')(context, {'id': pkg_id})
        except toolkit.ObjectNotFound:
            return False
    if package.get('state', '') != 'active' and package.get('state', '') != 'draft':
        return False
    resources = package.get('resources', [])
    if len(resources) == 0:
        return False
    if not any([r.get('state', '') == 'active' for r in resources]):
        return False
    if package.get('private', False):
        return False
    return True



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

from datetime import datetime as dt

from beaker.cache import CacheManager, cache_managers
from beaker.util import parse_cache_config_options
from ckanext.twitter.lib import config_helpers

cache_opts = {'cache.type': 'memory', 'cache.lock_dir': '/tmp/cache/lock'}

cache_manager = CacheManager(**parse_cache_config_options(cache_opts))
twitter_cache = cache_manager.get_cache('twitter')


def cache(pkg_id):
    """
    Adds the package id and current time to the 'twitter' cache. Clears any existing
    entries for the given package id first.

    :param pkg_id: The package id to store.
    """
    twitter_cache.remove_value(pkg_id)
    twitter_cache.put(pkg_id, dt.now())


def reset_cache():
    """
    Clears everything from the 'twitter' cache.
    """
    twitter_cache.clear()
    for k, c in cache_managers.items():
        c.clear()


def remove_from_cache(pkg_id):
    """
    Remove the package id from the cache.

    :param pkg_id:
    """
    twitter_cache.remove_value(pkg_id)


def expired(pkg_id):
    """
    Checks to see if the cache entry for the package's last tweet (if any) is old enough
    to be overwritten.

    :param pkg_id: The package ID.
    :return: boolean
    """
    if not twitter_cache.has_key(pkg_id):
        return True
    last_posted = twitter_cache.get(pkg_id)
    hours_since = (dt.now() - last_posted).seconds / 3600
    return hours_since > config_helpers.twitter_hours_between_tweets()



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

from beaker.cache import cache_regions
from ckan.common import session
from ckan.plugins import SingletonPlugin, implements, interfaces, toolkit

import ckanext.twitter.lib.config_helpers
from ckanext.twitter import routes
from ckanext.twitter.lib import config_helpers, helpers as twitter_helpers


class TwitterPlugin(SingletonPlugin):
    """
    Automatically send tweets when a dataset is updated or created.
    """

    implements(interfaces.IConfigurable, inherit=True)
    implements(interfaces.IConfigurer)
    implements(interfaces.IPackageController, inherit=True)
    implements(interfaces.ITemplateHelpers, inherit=True)
    implements(interfaces.IBlueprint, inherit=True)

    # IConfigurable
    def configure(self, config):
        cache_regions.update(
            {
                'twitter': {
                    'expire': ckanext.twitter.lib.config_helpers.twitter_hours_between_tweets(),
                    'type': 'memory',
                    'enabled': True,
                    'key_length': 250,
                }
            }
        )

    # IConfigurer
    def update_config(self, config):
        # Add templates
        toolkit.add_template_directory(config, 'theme/templates')
        # Add resources
        toolkit.add_resource('theme/assets', 'ckanext-twitter')

    # IPackageController
    def after_update(self, context, pkg_dict):
        is_suitable = twitter_helpers.twitter_pkg_suitable(context, pkg_dict['id'])
        if is_suitable:
            session.setdefault('twitter_is_suitable', pkg_dict['id'])
            session.save()

    # ITemplateHelpers
    def get_helpers(self):
        js_helpers = twitter_helpers.TwitterJSHelpers()
        return {
            'tweet_ready': js_helpers.tweet_ready,
            'get_tweet': js_helpers.get_tweet,
            'disable_edit': config_helpers.twitter_disable_edit,
        }

    ## IBlueprint
    def get_blueprint(self):
        return routes.blueprints



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

import math
import re

from ckan.lib.search import SearchIndexError
from ckan.plugins import toolkit
from ckanext.twitter.lib import config_helpers
from jinja2 import Environment

tweet_limit = 140


def extract_info(context, pkg_dict, template_length, tokens):
    """
    Creates a simplified dictionary for use in a tweet string template.

    :param context: The current context.
    :param pkg_dict: The package information to be simplified.
    :param template_length: The length of the text in the template (without any of the tokens).
    :param tokens: A list of token names in the template (extracted via regex).
    :return: dict
    """
    # get the values that are simple (i.e. not lists or dicts)
    simplified = {
        k: v
        for k, v in pkg_dict.items()
        if not isinstance(v, list) and not isinstance(v, dict) and v is not None
    }

    # turn the lists into counts
    list_lengths = {
        k: len(v)
        for k, v in pkg_dict.items()
        if isinstance(v, list) or isinstance(v, dict)
    }
    for k, v in list_lengths.items():
        simplified[k] = v

    # apply specific rules to certain fields
    simplified['records'] = get_number_records(context, pkg_dict['id'])
    simplified['author'] = truncate_author(simplified.get('author', 'Anon.'))

    # truncate other fields
    other_tokens = [
        t for t in tokens if t not in ['records', 'author'] and t in simplified.keys()
    ]
    max_total_token = tweet_limit - template_length
    total_token = sum(
        [len(str(simplified[t])) for t in tokens if t in ['records', 'author']]
    )
    for i in range(len(other_tokens)):
        char_limit = math.floor(
            (max_total_token - total_token) / (len(other_tokens) - i)
        )
        val = str(simplified.get(other_tokens[i], '')).strip()
        if len(val) > char_limit:
            val = truncate_field(val, char_limit)
        simplified[other_tokens[i]] = val
        total_token += len(val)
    return simplified


def truncate_author(author):
    """
    Shortens the author field using regular expressions.

    :param author: The full author string.
    :return: str
    """
    sep_rgx = '\s?[,;]\s?'
    name_sep_rgx = '(?<=[^,;])\s'
    separators = list(set(re.findall(sep_rgx, author)))
    name_sep = re.search(name_sep_rgx, author)
    if len(separators) == 0:
        return re.split(name_sep_rgx, author)[-1] if name_sep else author
    first_author = re.split(sep_rgx, author)[0]
    if len(separators) == 1:
        first_author = re.split(name_sep_rgx, first_author)[1]
    return f'{first_author} et al.'


def truncate_field(value, char_limit):
    """
    Shortens the given value to a length equal to or less than the character limit and
    appends a continuation marker.

    :param value: The value to be truncated.
    :param char_limit: The maximum number of characters in the output string.
    :return: str
    """
    marker = '[...]'
    truncated = []
    if ' ' in value:
        parts = value.split(' ')
        for p in parts:
            if sum([len(i) + 1 for i in truncated]) + len(p) + len(marker) < char_limit:
                truncated.append(p)
            else:
                break
        return ' '.join(truncated) + marker
    return value[: char_limit - len(marker)] + marker


def get_number_records(context, pkg_id):
    """
    Counts the total number of records associated with a package.

    :param context: The current context.
    :param pkg_id: The package ID.
    :return: int
    """
    pkg = toolkit.get_action('package_show')(context, {'id': pkg_id})
    resources = pkg.get('resources', None)
    if not resources or len(resources) == 0:
        return 0
    resource_ids = [r['id'] for r in resources]
    total = 0
    for rid in resource_ids:
        try:
            resource_data = toolkit.get_action('datastore_search')(
                context, {'resource_id': rid}
            )
            total += resource_data.get('total', 0)
        except (toolkit.ObjectNotFound, SearchIndexError):
            pass
    return total


def generate_tweet(context, pkg_id, is_new, force_truncate=True):
    """
    Generates a standard tweet based on template values in the config. Does not post the
    tweet; just generates and returns the text.

    :param context: The current context.
    :param pkg_id: The ID of the package to tweet about.
    :param is_new: True if the package has only just been created or given
    its first resource, False if it's being updated.
    :param force_truncate: If True, enforces an extra check at the end to
    ensure the text is below 140 characters. This should not be necessary as
    other methods account for this, but this is an optional final check.
    :return: str
    """
    pkg = toolkit.get_action('package_show')(context, {'id': pkg_id})
    if pkg.get('private', False):
        return
    format_string = (
        config_helpers.twitter_new_format()
        if is_new
        else config_helpers.twitter_updated_format()
    )
    tokens = re.findall('(?:{{ )(\w+)(?:(?:|.+?)? }})', format_string)
    template = Environment().from_string(format_string)
    simplified_dict = extract_info(context, pkg, len(str(template.module)), tokens)
    rendered = template.render(simplified_dict)
    # extra check to make sure the tweet isn't too long
    if len(rendered) > tweet_limit and force_truncate:
        rendered = rendered[:tweet_limit]
    return rendered



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK
from unittest.mock import patch, MagicMock

import pytest
from ckan.tests import factories
from ckan.tests.helpers import call_action

from ckanext.twitter.lib import parsers as twitter_parsers, twitter_api


@pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
@pytest.mark.ckan_config('ckan.plugins', 'datastore')
@pytest.mark.usefixtures(
    'clean_db', 'clean_datastore', 'with_plugins', 'with_request_context'
)
class TestTweetGeneration(object):
    def test_public(self):
        package = factories.Dataset()

        tweet_text = twitter_parsers.generate_tweet({}, package['id'], is_new=True)
        assert tweet_text is not None

    def test_private(self):
        # need an org to make a private package
        owner_org = factories.Organization()
        package = factories.Dataset(owner_org=owner_org['id'], private=True)

        # need to ignore the auth so that we can anonymously access the private package (we could
        # solve this by creating a user and adding them to the org above and then using that user
        # in the context, but this is easier tbh)
        context = {'ignore_auth': True}
        tweet_text = twitter_parsers.generate_tweet(context, package['id'], is_new=True)
        assert tweet_text is None

    @pytest.mark.ckan_config(
        'ckanext.twitter.new', '{{ title }} / {{ author }} / {{ records }}'
    )
    def test_custom_new_text(self):
        title = 'A package title'
        author = 'Author'

        package = factories.Dataset(title=title, author=author)
        resource = factories.Resource(package_id=package['id'], url_type='datastore')

        records = [{'x': i, 'y': f'number {i}'} for i in range(10)]
        call_action('datastore_create', resource_id=resource['id'], records=records)

        tweet_text = twitter_parsers.generate_tweet({}, package['id'], is_new=True)
        correct_tweet_text = f'{title} / {author} / {len(records)}'
        assert tweet_text == correct_tweet_text

    def test_default_new_text(self):
        title = 'A package title'
        author = 'Author'

        package = factories.Dataset(title=title, author=author)
        resource = factories.Resource(package_id=package['id'], url_type='datastore')

        records = [{'x': i, 'y': f'number {i}'} for i in range(10)]
        call_action('datastore_create', resource_id=resource['id'], records=records)

        tweet_text = twitter_parsers.generate_tweet({}, package['id'], is_new=True)
        correct_tweet_text = (
            f'New dataset: "{title}" by {author} ({len(records)} records).'
        )
        assert tweet_text == correct_tweet_text

    @pytest.mark.ckan_config(
        'ckanext.twitter.updated', '{{ title }} / {{ author }} / {{ records }}'
    )
    def test_custom_updated_text(self):
        title = 'A package title'
        author = 'Author'

        package = factories.Dataset(title=title, author=author)
        resource = factories.Resource(package_id=package['id'], url_type='datastore')

        records = [{'x': i, 'y': f'number {i}'} for i in range(10)]
        call_action('datastore_create', resource_id=resource['id'], records=records)

        tweet_text = twitter_parsers.generate_tweet({}, package['id'], is_new=False)
        correct_tweet_text = f'{title} / {author} / {len(records)}'
        assert tweet_text == correct_tweet_text

    def test_default_updated_text(self):
        title = 'A package title'
        author = 'Author'

        package = factories.Dataset(title=title, author=author)
        resource = factories.Resource(package_id=package['id'], url_type='datastore')

        records = [{'x': i, 'y': f'number {i}'} for i in range(10)]
        call_action('datastore_create', resource_id=resource['id'], records=records)

        tweet_text = twitter_parsers.generate_tweet({}, package['id'], is_new=False)
        correct_tweet_text = (
            f'Updated dataset: "{title}" by {author} ({len(records)} records).'
        )
        assert tweet_text == correct_tweet_text

    @pytest.mark.ckan_config('ckanext.twitter.debug', True)
    def test_does_not_tweet_when_debug(self):
        tweeted, reason = twitter_api.post_tweet('Mock text', MagicMock())
        assert not tweeted
        assert reason == 'debug'

    def test_shortens_author(self):
        title = 'A package title'
        author = 'Captain Author; Captain Author2; Captain Author3'

        package = factories.Dataset(title=title, author=author)

        tweet_text = twitter_parsers.generate_tweet({}, package['id'], is_new=True)
        correct_tweet_text = f'New dataset: "{title}" by Author et al. (0 resource).'
        assert tweet_text == correct_tweet_text

    def test_shortens_title(self):
        title = 'A package title that is pretty long but not ridiculously long, woo!'
        author = 'Captain Author'

        package = factories.Dataset(title=title, author=author)

        tweet_text = twitter_parsers.generate_tweet({}, package['id'], is_new=True)
        correct_tweet_text = f'New dataset: "{title[:43]}[...]" by Author (0 resource).'
        assert tweet_text == correct_tweet_text

    def test_does_not_exceed_140_chars(self):
        title = 'A package title that is pretty long but not ridiculously long, woo!'
        author = '; '.join(f'Captain Author{i}' for i in range(40))

        package = factories.Dataset(title=title, author=author)

        force_truncate = twitter_parsers.generate_tweet({}, package['id'], is_new=True)
        no_force = twitter_parsers.generate_tweet(
            {}, package['id'], is_new=True, force_truncate=False
        )
        assert len(force_truncate) <= 140
        assert len(no_force) <= 140

    @pytest.mark.ckan_config('debug', False)
    @pytest.mark.ckan_config('ckanext.twitter.debug', False)
    def test_does_not_tweet_when_recently_tweeted(self):
        # mock the expires function to always return that the mock package id we send in hasn't
        # expired yet
        mock_cache_helpers = MagicMock(expired=MagicMock(return_value=False))
        with patch('ckanext.twitter.lib.twitter_api.cache_helpers', mock_cache_helpers):
            tweeted, reason = twitter_api.post_tweet(
                'This is a test tweet.', MagicMock()
            )
        assert not tweeted
        assert reason == 'insufficient rest period'

    @pytest.mark.ckan_config('debug', False)
    @pytest.mark.ckan_config('ckanext.twitter.debug', False)
    def test_does_tweet_when_not_recently_tweeted(self):
        # this test also confirms the auth check works...

        mock_cache_helpers = MagicMock(expired=MagicMock(return_value=True))
        with patch('ckanext.twitter.lib.twitter_api.cache_helpers', mock_cache_helpers):
            tweeted, reason = twitter_api.post_tweet(
                'This is a test tweet.', MagicMock()
            )

        assert not tweeted
        assert reason == 'not authenticated'



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

from ckan.plugins import toolkit


def twitter_get_credentials():
    """
    Retrieves twitter API key and secret from config file.

    :return: (key, secret)
    """
    consumer_key = toolkit.config.get(
        'ckanext.twitter.consumer_key', 'no-consumer-key-set'
    )
    consumer_secret = toolkit.config.get(
        'ckanext.twitter.consumer_secret', 'no-consumer-secret-set'
    )
    token_key = toolkit.config.get('ckanext.twitter.token_key', 'no-token-key-set')
    token_secret = toolkit.config.get(
        'ckanext.twitter.token_secret', 'no-token-secret-set'
    )
    return consumer_key, consumer_secret, token_key, token_secret


def twitter_is_debug():
    '''
    Checks debug flags in the config - the plugin-specific flag can override
    the global debug flag.
    :return: boolean
    '''
    return toolkit.config.get(
        'ckanext.twitter.debug', toolkit.config.get('debug', False)
    )


def twitter_hours_between_tweets():
    """
    For calculating the 'rest period' between subsequent tweets about the same dataset.

    :return: int
    """
    return toolkit.config.get('ckanext.twitter.hours_between_tweets', 24)


def twitter_new_format():
    """
    Gets the string defining the format of the tweet that will be posted for new
    datasets.

    :return: string with replaceable jinja2 tags
    """
    return toolkit.config.get(
        'ckanext.twitter.new',
        'New dataset: "{{ title }}" by {{ author }} ({'
        '%- if records != 0 -%} {{ records }} records {'
        '%- else -%} {{ resources }} resource {%- endif '
        '-%}).',
    )


def twitter_updated_format():
    """
    Gets the string defining the format of the tweet that will be posted for updated.

    :return: string with replaceable jinja2 tags
    """
    return toolkit.config.get(
        'ckanext.twitter.updated',
        'Updated dataset: "{{ title }}" by {{ author }} '
        '({%- if records != 0 -%} {{ records }} records '
        '{%- elif resources == 1 -%} {{ resources }} '
        'resource {%- else -%} {{ resources }} '
        'resources {%- endif -%}).',
    )


def twitter_disable_edit():
    """
    Checks for a disable_edit flag in the config.

    If true, this prevents the
    user editing the tweet before it is posted.
    :return: boolean
    """
    return toolkit.config.get('ckanext.twitter.disable_edit', False)



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK
from unittest.mock import patch, MagicMock, call

import pytest
from ckan.model import State
from ckan.tests import factories
from ckan.tests.helpers import call_action

from ckanext.twitter.lib.helpers import TwitterJSHelpers, twitter_pkg_suitable


@pytest.fixture
def js_helpers():
    return TwitterJSHelpers()


@pytest.fixture
def session():
    return {}


class TestGetConfigVariables(object):
    def test_returns_false_if_not_in_session(self, js_helpers, session):
        package = factories.Dataset()

        with patch('ckanext.twitter.lib.helpers.session', session):
            assert not js_helpers.tweet_ready(package['id'])

    def test_returns_true_if_is_in_session(self, js_helpers, session):
        package = factories.Dataset()
        session['twitter_is_suitable'] = package['id']

        with patch('ckanext.twitter.lib.helpers.session', session):
            assert js_helpers.tweet_ready(package['id'])

    @pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
    @pytest.mark.usefixtures('clean_db', 'with_request_context')
    def test_gets_tweet(self, js_helpers):
        package = factories.Dataset()

        mock_parsers = MagicMock()

        with patch('ckanext.twitter.lib.helpers.twitter_parsers', mock_parsers):
            js_helpers.get_tweet(package['id'])

        assert mock_parsers.generate_tweet.call_args == call({}, package['id'], True)

    @pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
    @pytest.mark.usefixtures('clean_db', 'with_request_context')
    def test_gets_tweet_old_package(self, js_helpers):
        package = factories.Dataset()
        for i in range(10):
            call_action('package_patch', id=package['id'], notes=f'Note number {i}')

        mock_parsers = MagicMock()

        with patch('ckanext.twitter.lib.helpers.twitter_parsers', mock_parsers):
            js_helpers.get_tweet(package['id'])

        assert mock_parsers.generate_tweet.call_args == call({}, package['id'], False)

    @pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
    @pytest.mark.usefixtures('clean_db', 'with_request_context')
    def test_not_suitable_if_does_not_exist(self):
        is_suitable = twitter_pkg_suitable({}, 'not-a-real-id')
        assert not is_suitable

    @pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
    @pytest.mark.usefixtures('clean_db', 'with_request_context')
    def test_not_suitable_if_not_active(self):
        package = factories.Dataset(state='inactive')
        context = {'ignore_auth': True}
        is_suitable = twitter_pkg_suitable(context, package['id'])
        assert not is_suitable

    @pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
    @pytest.mark.usefixtures('clean_db', 'with_request_context')
    def test_not_suitable_if_no_resources(self):
        package = factories.Dataset(state=State.ACTIVE)

        is_suitable = twitter_pkg_suitable({}, package['id'])
        assert not is_suitable

    @pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
    @pytest.mark.usefixtures('clean_db', 'with_request_context')
    def test_not_suitable_if_no_active_resources(self):
        package = factories.Dataset(state=State.ACTIVE)
        resource = factories.Resource(package_id=package['id'])
        call_action('resource_delete', id=resource['id'])
        context = {'ignore_auth': True}
        is_suitable = twitter_pkg_suitable(context, package['id'])
        assert not is_suitable

    @pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
    @pytest.mark.usefixtures('clean_db', 'with_request_context')
    def test_not_suitable_if_private(self):
        # need an org to make a private package
        owner_org = factories.Organization()
        package = factories.Dataset(owner_org=owner_org['id'], private=True)
        factories.Resource(package_id=package['id'])

        # need to ignore the auth so that we can anonymously access the private package (we could
        # solve this by creating a user and adding them to the org above and then using that user
        # in the context, but this is easier tbh)
        context = {'ignore_auth': True}

        is_suitable = twitter_pkg_suitable(context, package['id'])
        assert not is_suitable

    @pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
    @pytest.mark.usefixtures('clean_db', 'with_request_context')
    def test_otherwise_suitable(self):
        # need an org to make a private package
        package = factories.Dataset()
        factories.Resource(package_id=package['id'])
        factories.Resource(package_id=package['id'])
        factories.Resource(package_id=package['id'])
        factories.Resource(package_id=package['id'])

        is_suitable = twitter_pkg_suitable({}, package['id'])
        assert is_suitable



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

import json

from ckan.common import session
from ckan.plugins import toolkit
from flask import Blueprint

from ckanext.twitter.lib import cache_helpers, twitter_api

blueprint = Blueprint(name='tweet', import_name=__name__)


@blueprint.route('/dataset/<package_id>/tweet', methods=['POST'])
def send(package_id):
    """
    Posts the tweet given in the request body. The package ID is required for caching.
    Returns json data for displaying success/error messages.

    :param package_id: The package ID (for caching).
    :return: str
    """
    body = toolkit.request.values
    text = body.get('tweet_text', None)
    if text:
        posted, reason = twitter_api.post_tweet(text, package_id)
    else:
        posted = False
        reason = 'no tweet defined'
    return json.dumps(
        {
            'success': posted,
            'reason': reason,
            'tweet': text if text else 'tweet not defined',
        }
    )


@blueprint.route('/dataset/<package_id>/tweet-clear', methods=['POST'])
def clear(package_id):
    cache_helpers.remove_from_cache(package_id)
    if 'twitter_is_suitable' in session:
        del session['twitter_is_suitable']
        session.save()
    return ''



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

from unittest.mock import patch, MagicMock

import pytest

from ckanext.twitter.lib import config_helpers
from ckanext.twitter.lib import twitter_api


@pytest.mark.ckan_config('ckanext.twitter.consumer_key', 'a-consumer-key')
@pytest.mark.ckan_config('ckanext.twitter.consumer_secret', 'a-consumer-secret')
@pytest.mark.ckan_config('ckanext.twitter.token_key', 'a-token-key')
@pytest.mark.ckan_config('ckanext.twitter.token_secret', 'a-token-secret')
def test_can_authenticate_success():
    mock_content = MagicMock()
    mock_response = MagicMock(status=200)
    mock_client = MagicMock(
        request=MagicMock(return_value=(mock_response, mock_content))
    )
    twitter_client_mock = MagicMock(return_value=mock_client)
    with patch('ckanext.twitter.lib.twitter_api.twitter_client', twitter_client_mock):
        is_authenticated = twitter_api.twitter_authenticate()
        assert is_authenticated


@pytest.mark.ckan_config('ckanext.twitter.consumer_key', 'a-consumer-key')
@pytest.mark.ckan_config('ckanext.twitter.consumer_secret', 'a-consumer-secret')
@pytest.mark.ckan_config('ckanext.twitter.token_key', 'a-token-key')
@pytest.mark.ckan_config('ckanext.twitter.token_secret', 'a-token-secret')
def test_can_authenticate_error():
    ck, cs, tk, ts = config_helpers.twitter_get_credentials()
    assert ck == 'a-consumer-key'
    assert cs == 'a-consumer-secret'
    assert tk == 'a-token-key'
    assert ts == 'a-token-secret'

    mock_content = MagicMock()
    mock_response = MagicMock(status=500)
    mock_client = MagicMock(
        request=MagicMock(return_value=(mock_response, mock_content))
    )
    twitter_client_mock = MagicMock(return_value=mock_client)
    with patch('ckanext.twitter.lib.twitter_api.twitter_client', twitter_client_mock):
        is_authenticated = twitter_api.twitter_authenticate()
        assert not is_authenticated


def test_can_authenticate_bad_creds():
    """
    Test what happens when we don't set any auth creds.
    """
    mock_content = MagicMock()
    mock_response = MagicMock(status=200)
    mock_client = MagicMock(
        request=MagicMock(return_value=(mock_response, mock_content))
    )
    twitter_client_mock = MagicMock(return_value=mock_client)
    with patch('ckanext.twitter.lib.twitter_api.twitter_client', twitter_client_mock):
        is_authenticated = twitter_api.twitter_authenticate()
        assert not is_authenticated



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

import pytest
from ckan.tests import factories
from ckan.tests.helpers import call_action
from unittest.mock import patch

from ckanext.twitter.lib import parsers as twitter_parsers


@pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
@pytest.mark.ckan_config('ckan.plugins', 'datastore twitter')
@pytest.mark.ckan_config('ckanext.twitter.debug', True)
@pytest.mark.usefixtures(
    'clean_db', 'clean_datastore', 'with_plugins', 'with_request_context'
)
@patch('ckanext.twitter.plugin.session')
class TestDatasetMetadata(object):
    def test_gets_dataset_number_of_records_if_has_records(self, mock_session):
        package = factories.Dataset()
        resource = factories.Resource(package_id=package['id'], url_type='datastore')

        records = [{'x': i, 'y': 'number {}'.format(i)} for i in range(10)]
        call_action('datastore_create', resource_id=resource['id'], records=records)

        record_count = twitter_parsers.get_number_records({}, package['id'])
        assert record_count == len(records)

    def test_gets_dataset_number_of_records_if_no_records(self, mock_session):
        package = factories.Dataset()
        resource = factories.Resource(package_id=package['id'], url_type='datastore')

        record_count = twitter_parsers.get_number_records({}, package['id'])
        assert record_count == 0



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-twitter
# Created by the Natural History Museum in London, UK

import json

import pytest
from ckan.plugins import toolkit
from ckan.tests import factories


@pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
@pytest.mark.ckan_config('ckan.plugins', 'twitter')
@pytest.mark.ckan_config('ckanext.twitter.debug', True)
@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestController(object):
    def test_url_created(self):
        url = toolkit.url_for('tweet.send', package_id='not-a-real-id')
        assert url == '/dataset/not-a-real-id/tweet'

    def test_url_ok(self, app):
        url = toolkit.url_for('tweet.send', package_id='not-a-real-id')
        response = app.post(url)
        assert response.status_code, 200

    def test_debug_post_tweet(self, app):
        dataset = factories.Dataset(notes='Test dataset')
        url = toolkit.url_for('tweet.send', package_id=dataset['id'])
        response = app.post(url, data={'tweet_text': 'this is a test tweet'})
        body = json.loads(response.body)
        assert body['reason'] == 'debug'
        assert body['tweet'] == 'this is a test tweet'
        assert not body['success']
